#!/env/python3
'''
Generates attachement for TST 010 based on index files in the api-tests repository

  Usage: python3 create_sols.py <path-to-api-tests>   <commit-or-tag>
'''

import os
import sys
import csv
import re

import subprocess

from robot.api import TestSuiteBuilder

import testspec
import config
import lib

indexes = {
    "SOL002" : 'sol_002_index.csv',
    "SOL003" : 'sol_003_index.csv',
    "SOL005" : 'sol_005_index.csv',
}

templates = {
    "SOL002" : 'Tests-SOL002-VeVnfm_template.docx',
    "SOL003" : 'Tests-SOL003-OrVnfm_template.docx',
    "SOL005" : 'Tests-SOL005-OsMaNfvo_template.docx'
}

outnames = {
    "SOL002" : 'Tests-SOL002-VeVnfm.docx',
    "SOL003" : 'Tests-SOL003-OrVnfm.docx',
    "SOL005" : 'Tests-SOL005-OsMaNfvo.docx',
}

def is_interface_heading(ref):
    '''Check if a reference has heading level 3'''
    return ref.lvl() == 3


def is_endpoint_heading(ref):
    '''Predicate to check if a reference has heading level 4'''
    return ref.lvl() == 4

def generate_new_clause(row, ref, spec, all_tests_root, commit_id, solspec, current_folder):
    '''
    Generates a new clause, either Void or for a Test Suite (i.e. a .robot file)
    '''
    if  "Void" in row["fln"]:
        lib.generate_void_clause(ref, row["title"], spec, solspec)
    else:
        path = os.path.join(all_tests_root, solspec, current_folder, row["fln"])
        suite = TestSuiteBuilder().build(path)
        lib.gen_api_suite(ref, suite, spec, current_folder, commit_id, solspec)

def load_spec_index(root_dir, base_spec):
    '''
    Load the index from the file, parses the CSV and applies strip 
    '''

    spec_idx = None

    index_filename = open(os.path.join(root_dir, "indexes", indexes[base_spec]), 'r')
    spec_idx = csv.DictReader(
        index_filename,
        skipinitialspace=True, 
        fieldnames=["ref", "title", "fln"]
    )

    if spec_idx is None:
        print("ERROR OPENING INDEX: {}".format(indexes[base_spec]))
        return []

    spec_idx = \
        (dict((k, v.strip()) for k, v in row.items() if v) for row in spec_idx if len(row) >= 3)

    return spec_idx

def create_test_for_sol(basedoc, spec_idx, all_tests_root, commit_id, output_dir):
    '''
    Loads index file for a base specification and iterates to produce 
    all clauses, 
    '''
    if not basedoc in indexes:
        print("Error: Wrong SOL Specification: {}".format(basedoc))
        return

    spec = testspec.TestSpec(templates[basedoc])
    current_folder = None

    for item in spec_idx:
        try:
            ref = lib.DocReference.from_str(item["ref"])

            if is_interface_heading(ref):
                spec.add_heading_from_ref(item["title"], ref)
                if "Void" in item["fln"]:
                    spec.add_void_paragraph()
                else:
                    current_folder = item["fln"]

            elif is_endpoint_heading(ref):
                if item["fln"] is None or current_folder is None:
                    print(item)
                    raise Exception()
                generate_new_clause(
                    item, ref, spec, all_tests_root, commit_id, basedoc, current_folder
                )

        except Exception as err:
            print(item, all_tests_root, basedoc, current_folder)
            raise err

    outfn = os.path.join(output_dir,outnames[basedoc])
    print("Writing {} definitions to: {}, ({})".format(basedoc, outfn, os.path.realpath(outfn)))
    spec.save(os.path.realpath(outfn))

def get_git_log(folder: str):
    '''
    Reads the oneline decorate log of the last commit of the HEAD in `folder`
    '''

    current_dir = os.path.realpath(os.curdir)
    os.chdir(folder)
    gitcmd = ['git', 'log', '-n', '1', '--oneline', '--decorate']
    
    gitlog = subprocess.run(gitcmd, check=True, stdout=subprocess.PIPE, text=True).stdout
    
    os.chdir(current_dir)

    return gitlog 

def parse_tag_from_log(gitlog):
    '''
    Returns the current git tag or None if the last commit is not tagged
    '''
    tag = re.search(r"(tag: )([^,]+)(,)", gitlog)

    return tag.group(2) if bool(tag) else None

def parse_commit_from_log(gitlog):
    '''
    Returns the current git tag or None if the last commit is not parsed
    '''
    commit = re.search(r"^([^\s]+)( )", gitlog)

    return commit.group(1) if bool(commit) else None

def get_git_reference(folder):
    '''
    Reads the tag or commit reference. Returns a couple (ref, reftype) where
    reftype is "tag" or "commit". If none can be parse, returns (None, None)
    '''

    git_log = get_git_log(folder) 
    tag = parse_tag_from_log(git_log)
    commit = parse_commit_from_log(git_log)

    if tag is not None:
        return (tag, "tag")
    
    if commit is not None:
        return (commit, "commit")

    print("Could not parse tag or commit locally. (Git log: '{}')".format(git_log))

    return (None, None)

if __name__ == "__main__":

    if len(sys.argv) < 2:
        print("Usage: robot2doc <sol_robot_root_dir> <commit-id> [<build-dir>]")
        print()
        print("       if <commit-id> is 'local', than attempts \n\
                      to read the reference from the git repository in the root dir")
        sys.exit(-1)

    ROOT = sys.argv[1]
    COMMIT_ID = sys.argv[2] if len(sys.argv) > 2 else None
    OUTDIR = sys.argv[3] if len(sys.argv) > 3 else ".."

    print("Received arguments: ", sys.argv)

    if COMMIT_ID == "local":
        (REF, REF_TYPE) = get_git_reference(ROOT)

        if REF is None:
            sys.exit(-1)
        COMMIT_ID = REF
        print("Using git ref: {} ({})".format(COMMIT_ID, REF_TYPE))


    print("Generating entire SOL tests")

    sol_specs = ["SOL002", "SOL003", "SOL005"]

    for solspec in sol_specs:
        spec_index = load_spec_index(ROOT, solspec)
        create_test_for_sol(solspec, spec_index, ROOT, COMMIT_ID, OUTDIR)

